/*
 * beaglefs/dir.c - The beaglefs directory object aka a list of inodes
 *
 * Robert Love <rml@novell.com>
 */

#include <string.h>

#include "dir.h"
#include "inode.h"

/* the hash table of inodes in the beaglefs */
static GHashTable *dir_hash;

/* the mutex that protects said hash AND its contents */
static GStaticMutex dir_mutex = G_STATIC_MUTEX_INIT;

/*
 * beagle_dir_get_count - Return the number of entries (hits) in the directory.
 */
unsigned long
beagle_dir_get_count (void)
{
	return g_hash_table_size (dir_hash);
}

void
beagle_dir_lock (void)
{
	g_static_mutex_lock (&dir_mutex);
}

void
beagle_dir_unlock (void)
{
	g_static_mutex_unlock (&dir_mutex);
}

/*
 * beagle_dir_get_inode - Return the inode matching the given name, or NULL if
 * no such inode exists.
 *
 * The caller must hold the dir lock.
 */
beagle_inode_t 
beagle_dir_get_inode (const char *name)
{
	g_return_val_if_fail (name, NULL);

	return g_hash_table_lookup (dir_hash, name);
}

/*
 * beagle_dir_for_each_inode - Invoke a function on each inode in the
 * directory.  The function matches the prototype
 *
 *	void f (gpointer key, gpointer value, gpointer user);
 *
 * Where 'key' is the path name of the inode, 'value' is an inode object, and
 * 'user' is the 'user' parameter provided to beagle_dir_for_each_inode().
 *
 * The caller must hold the dir lock.
 */
void
beagle_dir_for_each_inode (GHFunc func,
			   gpointer user)
{
	g_hash_table_foreach (dir_hash, func, user);
}

/*
 * beagle_dir_add_entry - Add the given inode object to the list of inodes.
 *
 * The caller must hold the dir lock.
 */
void
beagle_dir_add_entry (beagle_inode_t inode)
{
	char *key;

	g_return_if_fail (inode);

	key = g_strdup (beagle_inode_get_name (inode));
	g_hash_table_replace (dir_hash, key, inode);
}

/*
 * beagle_dir_remove_entries_by_name - Remove any entries from the inode list
 * that match the given name.  We do this twice, obviously when live-removing
 * hits, but also when adding hits, as we need to remove any duplicates.
 *
 * Caller must hold the dir lock.
 */
void
beagle_dir_remove_entries_by_name (const char *name)
{
	g_return_if_fail (name);
	g_hash_table_remove (dir_hash, name);
}

static void
value_destroy_func (void *data)
{
	beagle_inode_t inode = data;
	beagle_inode_free (inode);
}

/*
 * beagle_dir_init - Initialize the directory object.
 */
void
beagle_dir_init (void)
{
	dir_hash = g_hash_table_new_full (g_str_hash,
					  g_str_equal,
					  g_free,
					  value_destroy_func);
}
