/*
 * beaglefs/hit.c - Query hit processing engine
 *
 * Robert Love <rml@novell.com>
 */

#include <string.h>

#include <beagle/beagle.h>
#include <glib.h>

#include "dir.h"
#include "hit.h"
#include "inode.h"

char *query_string;

/*
 * hits_added_cb - Our callback for the libbeagle "hits-added" signal.  For
 * each hit, create a new inode object and add it to the filesystem.
 */
static void
hits_added_cb (G_GNUC_UNUSED BeagleQuery *query,
	       BeagleHitsAddedResponse *response) 
{
	GSList *hits, *elt;

	hits = beagle_hits_added_response_get_hits (response);
	for (elt = hits; elt; elt = g_slist_next (elt)) {
		beagle_inode_t inode;
		const char *name;

		inode = beagle_inode_new_from_hit (BEAGLE_HIT (elt->data));
		name = beagle_inode_get_name (inode);

		beagle_dir_lock ();
		beagle_dir_add_entry (inode);
		beagle_dir_unlock();
	}

	/* FIXME: Free or unref response and hits ? */
}

/*
 * hits_subtracted_cb - Our callback for the libbeagle "hits-subtracted"
 * signal.  For each hit, remove the corresponding inode from the filesystem.
 */
static void
hits_subtracted_cb (G_GNUC_UNUSED BeagleQuery *query,
		    BeagleHitsSubtractedResponse *response) 
{
	GSList *hits, *elt;

	hits = beagle_hits_subtracted_response_get_uris (response);
	for (elt = hits; elt; elt = g_slist_next (elt)) {
		const char *name = strrchr (elt->data, '/');

		beagle_dir_lock ();
		beagle_dir_remove_entries_by_name (name);
		beagle_dir_unlock ();
	}

	/* FIXME: Free or unref hits and response ? */
}

static void *
beagle_thread_start (G_GNUC_UNUSED void *ignored)
{
	BeagleClient *client;
	BeagleQuery *query;
	GMainLoop *main_loop;

	client = beagle_client_new (NULL);
	main_loop = g_main_loop_new (NULL, FALSE);
	query = beagle_query_new ();

	beagle_query_add_text (query, (const char *) query_string);
	beagle_query_add_hit_type (query, "File");
	beagle_query_add_hit_type (query, "IMLog");

	g_signal_connect (query,
			  "hits-added",
			  G_CALLBACK (hits_added_cb),
			  client);

	g_signal_connect (query,
			  "hits-subtracted",
			  G_CALLBACK (hits_subtracted_cb),
			  client);

	beagle_client_send_request_async (client, BEAGLE_REQUEST (query), NULL);

	g_main_loop_run (main_loop);
	g_object_unref (query);
	g_object_unref (client);
	g_main_loop_unref (main_loop);

	return NULL;
}

void
beagle_hit_init (void)
{
	g_thread_init (NULL);
	g_type_init ();

	g_thread_create (beagle_thread_start, NULL, FALSE, NULL);
}
