/*
 * beaglefs/inode.c - the beaglefs inode object
 *
 * Robert Love <rml@novell.com>
 */

#include <string.h>
#include <time.h>

#include <glib.h>
#include <beagle/beagle.h>

#include "inode.h"

struct beagle_inode {
	char *name;		/* filename */
	char *target;		/* target of the symlink, absolute file path */
	time_t time;		/* hit's m_time, c_time, and a_time */
	char *mime_type;	/* xattr: MIME Type */
	char *type;		/* xattr: Hit Type */
	char *uri;		/* xattr: URI */
	char *source;		/* xattr: Source */
	char *score;		/* xattr: Hit Score */
};

const char * beagle_inode_get_name (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->name;
}

const char * beagle_inode_get_target (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->target;
}

time_t * beagle_inode_get_time (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return &inode->time;
}

const char * beagle_inode_get_mime_type (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->mime_type;
}

const char * beagle_inode_get_type (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->type;
}

const char * beagle_inode_get_uri (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->uri;
}

const char * beagle_inode_get_source (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->source;
}

const char * beagle_inode_get_score (beagle_inode_t inode)
{
	g_return_val_if_fail (inode, NULL);
	return inode->score;
}

/*
 * beagle_inode_new_from_hit - create a new beaglefs inode object and
 * initialize its default values via a BeagleHit object
 *
 * Returns the newly allocated inode object, which must be freed via a call to
 * beagle_inode_free().
 */
beagle_inode_t 
beagle_inode_new_from_hit (BeagleHit *hit)
{
	beagle_inode_t inode;
	BeagleTimestamp *hit_time;
	const char *file;

	g_return_val_if_fail (hit, NULL);

	inode = g_new (struct beagle_inode, 1);

	file = beagle_hit_get_uri (hit) + 7;
	inode->name = g_strdup (strrchr (file, '/'));
	inode->target = g_strdup (file);

	hit_time = beagle_hit_get_timestamp (hit);
	if (!beagle_timestamp_to_unix_time (hit_time, &inode->time))
		time (&inode->time);

	inode->mime_type = g_strdup (beagle_hit_get_mime_type (hit));
	inode->type = g_strdup (beagle_hit_get_type (hit));
	inode->uri = g_strdup (beagle_hit_get_uri (hit));
	inode->source = g_strdup (beagle_hit_get_source (hit));
	inode->score = g_strdup_printf ("%lf", beagle_hit_get_score (hit));

	return inode;
}

/*
 * beagle_inode_free - free an inode object and its constituents
 */
void
beagle_inode_free (beagle_inode_t inode)
{
	g_return_if_fail (inode);

	g_free (inode->score);
	g_free (inode->source);
	g_free (inode->uri);
	g_free (inode->type);
	g_free (inode->mime_type);
	g_free (inode->target);
	g_free (inode->name);

	g_free (inode);
}
